#include "playername.h"

inline void strlower( char * s )
{
	if(s && *s)
		do{
			unsigned char c = *(unsigned char*)s;
			if(c >= 'A' && c <= 'Z')
				*(unsigned char*)s = (unsigned char)(c + ('a' - 'A'));
		}while(*(++s));
}

inline unsigned short UTIL_FixedUnsigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);
	if ( output < 0 )
		output = 0;
	else if ( output > 0xFFFF )
		output = 0xFFFF;

	return (unsigned short)output;
}


inline short UTIL_FixedSigned16( float value, float scale )
{
	int output;

	output = (int)(value * scale);

	if ( output > 32767 )
		output = 32767;
	else if ( output < -32768 )
		output = -32768;

	return (short)output;
}


inline unsigned char UTIL_FixedUnsigned8( float value, float scale )
{
	int output = (int)(value*scale);
	
	if( output > 255 ) 
		output = 255;
	else if( output < 0 ) 
		output = 0;
	
	return (unsigned char)output;
}


void UTIL_LogPrintf( char *fmt, ... )
{
	va_list			argptr;
	static char		string[1024+1];
	
	va_start ( argptr, fmt );
	vsnprintf( string, 1024, fmt, argptr );
	va_end   ( argptr );

	string[1024] = 0;
	// Print to server console
	ALERT( at_logged, "%s", string );
}


inline char* UTIL_VarArgs( char *format, ... )
{
	va_list		argptr;
	
	static char	string[1024+1];
	
	va_start  (argptr, format);
	vsnprintf (string, 1024, format, argptr);
	va_end    (argptr);
	
	string[1024] = 0;
	return string;
}


void UTIL_SendHudMessageOne(edict_t *pEntity, const hudtextparms_t &textparms, const char *pMessage)
{
	if(!pEntity || !pMessage || !*pMessage) return;
	
	MESSAGE_BEGIN( MSG_ONE, SVC_TEMPENTITY, NULL, pEntity );
		WRITE_BYTE( TE_TEXTMESSAGE );
		WRITE_BYTE( textparms.channel & 0xFF );

		WRITE_SHORT( UTIL_FixedSigned16( textparms.x, 1<<13 ) );
		WRITE_SHORT( UTIL_FixedSigned16( textparms.y, 1<<13 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.effect, 1 ) );

		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.r1, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.g1, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.b1, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.a1, 1 ) );

		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.r2, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.g2, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.b2, 1 ) );
		WRITE_BYTE( UTIL_FixedUnsigned8( textparms.a2, 1 ) );

		WRITE_SHORT( UTIL_FixedUnsigned16( textparms.fadeinTime, 1<<8 ) );
		WRITE_SHORT( UTIL_FixedUnsigned16( textparms.fadeoutTime, 1<<8 ) );
		WRITE_SHORT( UTIL_FixedUnsigned16( textparms.holdTime, 1<<8 ) );

		if ( textparms.effect == 2 )
			WRITE_SHORT( UTIL_FixedUnsigned16( textparms.fxTime, 1<<8 ) );
		
		char sendMessage[MAX_HUD_TEXT_LENGTH];
		
		STRNCPY(sendMessage, pMessage, MAX_HUD_TEXT_LENGTH);
		
		sendMessage[MAX_HUD_TEXT_LENGTH-1] = 0;
		
		//There is some chars that aren't shows right way .. only two of these can be fixed :/
		for(int i = 0, slen = strlen(sendMessage); i < slen; i++)
		{
			switch( ((unsigned char*)sendMessage)[i] )
			{
				case ';':
					((unsigned char*)sendMessage)[i] = 60;
				break;
			
				case '<':
					((unsigned char*)sendMessage)[i] = 127;
				break;
			
				case '=':
				case '>':
				case '\\':
				case '^':
					//No fix for these yet :(
				break;
			}
		}
		
		WRITE_STRING( sendMessage );

	MESSAGE_END();
}


edict_t *UTIL_FindEntityForward( edict_t *pMe )
{
	if(!pMe) return NULL;
	
	TraceResult tr;

	MAKE_VECTORS(pMe->v.v_angle);
	UTIL_TraceLine(pMe->v.origin + pMe->v.view_ofs,pMe->v.origin + pMe->v.view_ofs + gpGlobals->v_forward * 8192,dont_ignore_monsters,ignore_glass, pMe, &tr );
	
	if ( tr.flFraction != 1.0 && !FNullEnt( tr.pHit) && IsPlayer(tr.pHit))
	{
		bool bInWater = (POINT_CONTENTS (tr.pHit->v.origin + tr.pHit->v.view_ofs) == CONTENTS_WATER);
		bool bLookerInWater = (POINT_CONTENTS (pMe->v.origin + pMe->v.view_ofs) == CONTENTS_WATER);
		
		if (bInWater != bLookerInWater)
      		{
      			return NULL;
      		}
		
		return tr.pHit;
	}
	
	return NULL;
}


inline void UTIL_GetColorValueFromCVars( t_color &color, cvar_t* color_cvar, const t_color &default_color )
{
	unsigned long r = 0, g = 0, b = 0;
	
	if( sscanf( color_cvar->string, "%lu:%lu:%lu", &r, &g, &b ) == 3 )
	{
		color.r = UTIL_FixedUnsigned8( r, 1 );
		color.g = UTIL_FixedUnsigned8( g, 1 );
		color.b = UTIL_FixedUnsigned8( b, 1 );
	}
	else
	{
		color.r = color.g = color.b = 0;
	}
	
	if( !color.r && !color.g && !color.b )
	{
		color = default_color;
		
		r = color.r;
		g = color.g;
		b = color.b;
		
		CVAR_DIRSET_STRING(color_cvar, UTIL_VarArgs( "%lu:%lu:%lu", r, g, b ) );
	}
}


void UTIL_GetColor( t_color &color, const color_type type )
{
	cvar_t* color_cvar = NULL;
	t_color default_color;
	
	switch ( type )
	{
	case color_Team1:
		color_cvar = team1_color;
		default_color = defDMColor;
	break;
	
	case color_Team2:
		color_cvar = team2_color;
		default_color = defEnemyColor;
	break;
	
	case color_Team3:
		color_cvar = team3_color;
		default_color = defTeam3Color;
	break;
		
	case color_Team4:
		color_cvar = team4_color;
		default_color = defFriendlyColor;
	break;

	case color_Friendly:
		color_cvar = friendly_color;
		default_color = defFriendlyColor;
	break;

	case color_Enemy:
		color_cvar = enemy_color;
		default_color = defEnemyColor;
	break;

	case color_DM:
	default:
		color_cvar = dm_color;
		default_color = defDMColor;
	}
	
	UTIL_GetColorValueFromCVars( color, color_cvar, default_color );
}


void MODCheck(void)
{
	mod_id = 0;
	
	int pos;
	
	char game_dir[512]; 
	memset(game_dir, 0, sizeof(game_dir));
	
	// find the directory name of the currently running MOD...
	(*g_engfuncs.pfnGetGameDir)(game_dir);

	pos = strlen(game_dir) - 1;

	// scan backwards till first directory separator...
	while ((pos) && (game_dir[pos] != '/'))
		pos--;

	if (pos) pos++;
	
	STRNCPY(mod_name, &game_dir[pos], 32);
	
	if (
		!stricmp(mod_name, "valve") 
			|| 
		!stricmp(mod_name, "dmc")//isn't really needed
			|| 
		!stricmp(mod_name, "phineas")
			||
		!stricmp(mod_name, "svencoop")//isn't really needed
			||
		!stricmp(mod_name, "ag")
			||
		!stricmp(mod_name, "oz")
	   )
	{
		mod_id=1;
		default_showenemy_on=true;
	}
	else if (!stricmp(mod_name, "tfc"))
	{
		mod_id=2;
		default_showenemy_on=true;
	}
	else if (!stricmp(mod_name, "dod"))
   	{
   		mod_id=2;
   	}
   	else if (!stricmp(mod_name, "cstrike"))
	{
		mod_id=3;
		default_showenemy_on=true;
	}
	else if (!stricmp(mod_name, "gearbox"))//have to be disabled for ctf
	{
		mod_id=4;
		default_showenemy_on=true;
	}
	
	return;
}


// half life multiplay has a simple concept of Player Relationships.
// you are either on another player's team, or you are not.
inline bool IsSameTeam_Method1(edict_t *pPlayer, edict_t *pTarget)
{
	const char *PlayerModel=GetModel(pPlayer);
	const char *TargetModel=GetModel(pTarget);
	
	return ( PlayerModel && TargetModel && *PlayerModel && *TargetModel && !stricmp( PlayerModel, TargetModel ) );
}


inline int GetTeamID_CS(edict_t *pEntity)
{
	char model_name[512];
	const char * temp;
	strcpy(model_name, ((temp = GetModel(pEntity))==0)?"":temp);
	strlower(model_name);
	
	if(!model_name || !model_name[0] || !model_name[1]) return 3;

	if ((int)(fastdetection->value)==0)
	{
		if ((!strcmp(model_name, "urban")) ||  // Seal Team 6
		    (!strcmp(model_name, "gsg9"))  ||  // German GSG-9
		    (!strcmp(model_name, "sas"))   ||  // UK SAS
		    (!strcmp(model_name, "gign"))  ||  // French GIGN
		    (!strcmp(model_name, "vip")) )     // VIP
		{
			return 1; //Blue Team
		} 
		else if ((!strcmp(model_name, "terror"))   ||  // Phoenix Connektion
		         (!strcmp(model_name, "arab"))     ||  // old L337 Krew
		    	 (!strcmp(model_name, "leet"))     ||  // L337 Krew
		    	 (!strcmp(model_name, "arctic"))   ||  // Arctic Avenger 
		    	 (!strcmp(model_name, "guerilla")) )   // Gorilla Warfare
		{
			return 2; //Red Team
		}
		
		return 3;  // return "YELLOW" if team is unknown
	}
	else
	{
		if (model_name[0]=='g')
		{
			if(model_name[1]=='u')
			{
				return 2; //Red Team
			}
			else
			{
				return 1; //Blue Team
			}
		}
		else if (model_name[0]=='a' || model_name[0]=='t' || model_name[0]=='l')
		{
			return 2; //Red Team
		} 
		else
		{
			return 1; //Blue Team
		}
	}
}


inline int GetTeamID_Op4CTF(edict_t *pEntity)
{
   	// OpFor CTF m...
	char model_name[512];
	const char * temp;
	strcpy(model_name, ((temp = GetModel(pEntity))==0)?"":temp);
	strlower(model_name);
	
	if(!model_name || !model_name[0]) return 1;

	if ((int)(fastdetection->value)==0)
   	{
   		if (	(!strcmp(model_name, "ctf_barney")) 
		      ||(!strcmp(model_name, "cl_suit")) 
		      ||(!strcmp(model_name, "ctf_gina")) 
		      ||(!strcmp(model_name, "ctf_gordon")) 
		      ||(!strcmp(model_name, "otis")) 
		      ||(!strcmp(model_name, "ctf_scientist")) )
		{ 
   			return 3; //"Yellow"-team
   		}
   	   	else if ((!strcmp(model_name, "beret")) 
		       ||(!strcmp(model_name, "drill")) 
        	       ||(!strcmp(model_name, "grunt")) 
        	       ||(!strcmp(model_name, "recruit")) 
        	       ||(!strcmp(model_name, "shephard")) 
        	       ||(!strcmp(model_name, "tower")) )
   		{ 
   			return 4; //"Green"-team
   		}
   		
   		return 1; //Red
   	}
   	else
   	{
   		if (model_name[0]=='c' || model_name[0]=='o')
   		{
   			return 3; //"Yellow"-team
   		} 	
   		else
   		{
   			return 4; //"Green"-team
   		}
  	}
}


bool Friendly_or_Enemy(const int teams, edict_t *pThis, edict_t *pEntity)
{
	switch(teams)
	{
		case 1:
			//VALVE
			//Simple model check
			return IsSameTeam_Method1(pThis,pEntity);
	
		case 2:
			//TFC && DOD
			//Check if is in same team then check allies.
			return (pThis->v.team==pEntity->v.team || (team_allies[(pThis->v.team-1)] & (1<<(pEntity->v.team-1))));

		case 3:
			//CS
			return (GetTeamID_CS(pThis)==GetTeamID_CS(pEntity));
	
		case 4:
			//GEARBOX
			return (is_defaultctf) ?
				(GetTeamID_Op4CTF(pThis)==GetTeamID_Op4CTF(pEntity))
				:
				(IsSameTeam_Method1(pThis,pEntity));
		default:
			return false;
	}
}


bool GetTeamColor(const int _mod_id, edict_t *pEntity, t_color &TextColor)
{
	//Have to be tfc or cs for fixed team colors!
	if (fixed_teams->value==0 || (_mod_id!=2 && _mod_id!=3)) return false;
	
	//Set yellow-team as default
	int targetteam=3;
	
	if (_mod_id==2)
		//TFC & DOD
		targetteam=pEntity->v.team;
		
	else if (_mod_id==3)
		//CS
		targetteam=GetTeamID_CS(pEntity);

	switch(targetteam)
	{
		case 1:
			//Set blue color
			UTIL_GetColor( TextColor, color_Team1 );
		break;
	
		case 2:
			//Set red color
			UTIL_GetColor( TextColor, color_Team2 );
		break;
	
		case 4:
			//Set green color
			UTIL_GetColor( TextColor, color_Team4 );
		break;

		default:
			//Set yellow color
			UTIL_GetColor( TextColor, color_Team3 );
		break;
	}

	return true;
}


//based on UTIL_FillText of clanmod
static char *UTIL_GetHudTextStyle( const char* format, edict_t * pEntity )
{
	if(!format) return "";
	
	static char string[1024+6]; 
	char slot_name[128+6];
	char *string_pos = string;
	char c = *format;
	unsigned int slot_len = 0;
	unsigned int string_len = 0;
	
	*string = 0;

	do {
		if( c == '%' && *(format+1) == '%' ) 
		{
			c = *(++format);
			
			goto MainLoopEnd;
		}
		else if( c != '%' )
		{
			goto MainLoopEnd;
		}
		
		slot_len	= 0;
		*string_pos	= 0;
		*slot_name	= 0;

		//get data name from "something%data_name%something"
		do {
			format++;
			c = *format;
			
			if( c == '%' || !c || slot_len > 128 )
			{
				if( c == '%' )
				{
					format++;
					c = *format;	
				}
				
				slot_name[slot_len] = 0;
				break;
			}
			
			slot_name[slot_len++] = c;
		} while( 1 );

		//Not slot_name found.. jump to end
		if( !*slot_name ) goto MainLoopEnd;

		//check for different supported info
		if( !strcmp( slot_name, "name" ) )
		{
			STRNCPY( string_pos, STRING(pEntity->v.netname), 1024 - string_len );
		}
		else if( !strcmp( slot_name, "health" ) )
		{
			snprintf( string_pos, 1024 - string_len, "%li", (signed long)(pEntity->v.health) );
		}
		else if( !strcmp( slot_name, "armor" ) )
		{
			snprintf( string_pos, 1024 - string_len, "%li", (signed long)(pEntity->v.armorvalue) );
		}
		
		//find end of string
		while(*(++string_pos)) 
		{
			string_len++;
		}

MainLoopEnd:
		if( !c )
		{
			break;
		}
		else if (c == '\\' && *(format + 1))
		{
			switch(*(format + 1))
			{
			case 't' : *(string_pos++) = '\t'; break;
			case 'n' : *(string_pos++) = '\n'; break;
			case '\\': *(string_pos++) = '\\'; break;
			default  : *(string_pos++) = *(format + 1); break;
			}
			
			*string_pos = 0;
			string_len++;
		
			format++; format++;
			c = *format;			
		}
		else
		{
			*(string_pos++) = c;
			*string_pos = 0;
			string_len++;

			format++;
			c = *format;
		}

	} while( c && string_len < 1024 );
	
	return string;
}


void GetFriendlyHudText(char *HUD_Text, const unsigned HUD_Text_max, edict_t *pEntity, int extrainfo)
{
	if (extrainfo>=1)
	{
		//Set default if null string
		if ( !*f_style1->string ) CVAR_DIRSET_STRING(f_style1, friendly_extra_style.string);
		
		//New style
		STRNCPY(HUD_Text, UTIL_GetHudTextStyle( f_style1->string, pEntity ), HUD_Text_max);
	}
	else
	{
		STRNCPY(HUD_Text, STRING(pEntity->v.netname), HUD_Text_max);
	}
}


void GetEnemyHudText(char *HUD_Text, const unsigned HUD_Text_max, edict_t *pEntity, int extrainfo)
{
	if (extrainfo>=3)
	{
		//Set default if null string
		if ( !*e_style2->string ) CVAR_DIRSET_STRING( e_style2, enemy_extra2_style.string );
		
		//New style
		STRNCPY(HUD_Text, UTIL_GetHudTextStyle( e_style2->string, pEntity ), HUD_Text_max);
	}
	else if (extrainfo>=1)
	{
		//Set default if null string
		if (!*e_style1->string) CVAR_DIRSET_STRING( e_style1, enemy_extra1_style.string );
		
		//New style
		STRNCPY(HUD_Text, UTIL_GetHudTextStyle( e_style1->string, pEntity ), HUD_Text_max);
	}
	else
	{
		STRNCPY(HUD_Text, STRING(pEntity->v.netname), HUD_Text_max);
	}	
}


void GetDMHudText(char *HUD_Text, const unsigned HUD_Text_max, edict_t *pEntity, int extrainfo)
{
	if (extrainfo>=2)
	{
		//Set default if null string
		if (!*dmstyle1->string) CVAR_DIRSET_STRING( dmstyle1, dm_extra_style.string );
		
		//New style
		STRNCPY(HUD_Text, UTIL_GetHudTextStyle( dmstyle1->string, pEntity ), HUD_Text_max);
	}
	else
	{
		STRNCPY(HUD_Text, STRING(pEntity->v.netname), HUD_Text_max);
	}
}


void CheckTeamPlay(void)
{
	if (	
		(mod_id==2)//TFC&DOD
		|| 
		(mod_id==3)//CS
		||
		(
			(mod_id==4)//OPFOR
				&&
			(is_defaultctf)//CTF
		)
	   ) 
	{
		TeamPlayIsOn = true;
	}
	else
	{
		TeamPlayIsOn = mp_teamplay_WasSetOnWorldSpawn;
	}
	
	TeamPlayChecked = true;
}
